@@ -1,7 +1,7 @@
 """ Module for image processing core methods
 
 .. include common links, assuming primary doc root is up one directory
-.. include:: ../links.rst
+.. include:: ../include/links.rst
 """
 import numpy as np
 from scipy import signal, ndimage
@@ -270,11 +270,19 @@ def rn_frame(datasec_img, gain, ronoise):
 
     Parameters
     ----------
+    datasec_img : ndarray
+        Index image (2D array) assigning each pixel in the data section to an amplifier.
+        A value of 0 means not a data section
+    gain : ndarray, list
+        A list of the gains for each amplifier
+    ronoise : ndarray, list
+        A list of read noise values for each amplifier. If any element of the array is 0.0,
+        the read noise will be determined from the overscan region
 
     Returns
     -------
     rn_img : ndarray
-      Read noise *variance* image (i.e. RN**2)
+        Read noise *variance* image (i.e. RN**2)
     """
     # Determine the number of amplifiers from the datasec image
     numamplifiers = np.amax(datasec_img)
@@ -312,15 +320,12 @@ def rect_slice_with_mask(image, mask, mask_val=1):
         mask_val (int,optiona): Value to mask on
 
     Returns:
-        np.ndarray, list:  Image at mask values, slices describing the mask
-
+        :obj:`tuple`: The image at mask values and a 2-tuple with the
+        :obj:`slice` objects that select the masked data.
     """
     pix = np.where(mask == mask_val)
-    slices = [slice(np.min(pix[0]), np.max(pix[0])+1),
-              slice(np.min(pix[1]), np.max(pix[1])+1)]
-    sub_img = image[tuple(slices)]
-    #
-    return sub_img, slices
+    slices = (slice(np.min(pix[0]), np.max(pix[0])+1), slice(np.min(pix[1]), np.max(pix[1])+1))
+    return image[slices], slices
 
 
 def subtract_overscan(rawframe, datasec_img, oscansec_img,
@@ -392,7 +397,7 @@ def subtract_overscan(rawframe, datasec_img, oscansec_img,
             raise ValueError('Unrecognized overscan subtraction method: {0}'.format(method))
 
         # Subtract along the appropriate axis
-        no_overscan[tuple(data_slice)] -= (ossub[:, None] if compress_axis == 1 else ossub[None, :])
+        no_overscan[data_slice] -= (ossub[:, None] if compress_axis == 1 else ossub[None, :])
 
     return no_overscan
 
@@ -520,7 +525,7 @@ def subtract_pattern(rawframe, datasec_img, oscansec_img, frequency=None, axis=1
                 msgs.warn("Pattern subtraction fit failed for row {0:d}/{1:d}".format(ii + 1, overscan.shape[0]))
                 continue
             model_pattern[ii, :] = cosfunc(xdata_all, *popt)
-        outframe[tuple(osd_slice)] -= model_pattern
+        outframe[osd_slice] -= model_pattern
 
     debug = False
     if debug:
@@ -692,10 +697,10 @@ def subtract_overscan(rawframe, numamplifiers, datasec, oscansec, method='savgol
             msgs.error('Overscan sections do not match amplifier sections for'
                        'amplifier {0}'.format(i+1))
         compress_axis = 1 if data_shape[0] == overscan.shape[0] else 0
-        
+
         # Fit/Model the overscan region
         osfit = np.median(overscan) if method.lower() == 'median' \
-                        else np.median(overscan, axis=compress_axis) 
+                        else np.median(overscan, axis=compress_axis)
         if method.lower() == 'polynomial':
             # TODO: Use np.polynomial.polynomial.polyfit instead?
             c = np.polyfit(np.arange(osfit.size), osfit, params[0])
