@@ -1,42 +1,43 @@
-""" Module for Shane/Kast specific codes
 """
-import numpy as np
+Module for Shane/Kast specific methods.
+
+.. include:: ../include/links.rst
+"""
 import os
 from pkg_resources import resource_filename
 
+from IPython import embed
+
+import numpy as np
+
 from astropy.time import Time
-from astropy.io import fits
 
 from pypeit import msgs
 from pypeit import telescopes
 from pypeit.core import framematch
-from pypeit.par import pypeitpar
 from pypeit.spectrographs import spectrograph
 from pypeit.images import detector_container
 from pypeit.core import parse
 
-from IPython import embed
+
 
 class ShaneKastSpectrograph(spectrograph.Spectrograph):
     """
     Child to handle Shane/Kast specific code
     """
     ndet = 1
+    telescope = telescopes.ShaneTelescopePar()
 
-    def __init__(self):
-        # Get it started
-
-        super(ShaneKastSpectrograph, self).__init__()
-        self.spectrograph = 'shane_kast'
-        self.telescope = telescopes.ShaneTelescopePar()
-        #self.timeunit = 'isot'
-
-    @staticmethod
-    def default_pypeit_par():
+    @classmethod
+    def default_pypeit_par(cls):
         """
-        Set default parameters for Shane Kast reductions.
+        Return the default parameters to use for this instrument.
+        
+        Returns:
+            :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
+            all of ``PypeIt`` methods.
         """
-        par = pypeitpar.PypeItPar()
+        par = super().default_pypeit_par()
 
         # Ignore PCA
         par['calibrations']['slitedges']['sync_predict'] = 'nearest'
@@ -56,49 +57,63 @@ class ShaneKastSpectrograph(spectrograph.Spectrograph):
         return par
 
     def compound_meta(self, headarr, meta_key):
+        """
+        Methods to generate metadata requiring interpretation of the header
+        data, instead of simply reading the value of a header card.
+
+        Args:
+            headarr (:obj:`list`):
+                List of `astropy.io.fits.Header`_ objects.
+            meta_key (:obj:`str`):
+                Metadata keyword to construct.
+
+        Returns:
+            object: Metadata value read from the header(s).
+        """
         if meta_key == 'mjd':
             time = headarr[0]['DATE']
             ttime = Time(time, format='isot')
             return ttime.mjd
-        else:
-            msgs.error("Not ready for this compound meta")
+        msgs.error("Not ready for this compound meta")
 
     def init_meta(self):
         """
-        Generate the meta data dict
-        Note that the children can add to this
-
-        Returns:
-            self.meta: dict (generated in place)
+        Define how metadata are derived from the spectrograph files.
 
+        That is, this associates the ``PypeIt``-specific metadata keywords
+        with the instrument-specific header cards using :attr:`meta`.
         """
-        meta = {}
+        self.meta = {}
         # Required (core)
-        meta['ra'] = dict(ext=0, card='RA')
-        meta['dec'] = dict(ext=0, card='DEC')
-        meta['target'] = dict(ext=0, card='OBJECT')
+        self.meta['ra'] = dict(ext=0, card='RA')
+        self.meta['dec'] = dict(ext=0, card='DEC')
+        self.meta['target'] = dict(ext=0, card='OBJECT')
         # dispname is arm specific (blue/red)
-        meta['decker'] = dict(ext=0, card='SLIT_N')
-        meta['binning'] = dict(ext=0, card=None, default='1,1')
-        meta['mjd'] = dict(ext=0, card=None, compound=True)
-        meta['exptime'] = dict(ext=0, card='EXPTIME')
-        meta['airmass'] = dict(ext=0, card='AIRMASS')
+        self.meta['decker'] = dict(ext=0, card='SLIT_N')
+        self.meta['binning'] = dict(ext=0, card=None, default='1,1')
+        self.meta['mjd'] = dict(ext=0, card=None, compound=True)
+        self.meta['exptime'] = dict(ext=0, card='EXPTIME')
+        self.meta['airmass'] = dict(ext=0, card='AIRMASS')
         # Additional ones, generally for configuration determination or time
-        meta['dichroic'] = dict(ext=0, card='BSPLIT_N')
+        self.meta['dichroic'] = dict(ext=0, card='BSPLIT_N')
         lamp_names = [ '1', '2', '3', '4', '5',
                        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K']
         for kk,lamp_name in enumerate(lamp_names):
-            meta['lampstat{:02d}'.format(kk+1)] = dict(ext=0, card='LAMPSTA{0}'.format(lamp_name))
-        # Ingest
-        self.meta = meta
+            self.meta['lampstat{:02d}'.format(kk+1)] = dict(ext=0, card='LAMPSTA{0}'.format(lamp_name))
 
     def configuration_keys(self):
         """
-        Set the configuration keys
+        Return the metadata keys that define a unique instrument
+        configuration.
+
+        This list is used by :class:`~pypeit.metadata.PypeItMetaData` to
+        identify the unique configurations among the list of frames read
+        for a given reduction.
 
         Returns:
-            cfg_keys: list
-
+            :obj:`list`: List of keywords of data pulled from file headers
+            and used to constuct the :class:`~pypeit.metadata.PypeItMetaData`
+            object.
         """
         # decker is not included because arcs are often taken with a 0.5" slit
         return ['dispname', 'dichroic' ]
@@ -106,13 +121,25 @@ class ShaneKastSpectrograph(spectrograph.Spectrograph):
     def check_frame_type(self, ftype, fitstbl, exprng=None):
         """
         Check for frames of the provided type.
+
+        Args:
+            ftype (:obj:`str`):
+                Type of frame to check. Must be a valid frame type; see
+                frame-type :ref:`frame_type_defs`.
+            fitstbl (`astropy.table.Table`_):
+                The table with the metadata for one or more frames to check.
+            exprng (:obj:`list`, optional):
+                Range in the allowed exposure time for a frame of type
+                ``ftype``. See
+                :func:`pypeit.core.framematch.check_frame_exptime`.
+
+        Returns:
+            `numpy.ndarray`_: Boolean array with the flags selecting the
+            exposures in ``fitstbl`` that are ``ftype`` type frames.
         """
         good_exp = framematch.check_frame_exptime(fitstbl['exptime'], exprng)
         if ftype in ['science', 'standard']:
             return good_exp & self.lamps(fitstbl, 'off')
-#            \
-#                        & np.array([ t not in ['Arcs', 'Bias', 'Dome Flat']
-#                                        for t in fitstbl['target']])
         if ftype == 'bias':
             return good_exp # & (fitstbl['target'] == 'Bias')
         if ftype in ['pixelflat', 'trace', 'illumflat']:
@@ -132,14 +159,15 @@ class ShaneKastSpectrograph(spectrograph.Spectrograph):
         Check the lamp status.
 
         Args:
-            fitstbl (:obj:`astropy.table.Table`):
+            fitstbl (`astropy.table.Table`_):
                 The table with the fits header meta data.
             status (:obj:`str`):
-                The status to check.  Can be `off`, `arcs`, or `dome`.
-        
+                The status to check. Can be ``'off'``, ``'arcs'``, or
+                ``'dome'``.
+
         Returns:
-            numpy.ndarray: A boolean array selecting fits files that
-            meet the selected lamp status.
+            `numpy.ndarray`_: A boolean array selecting fits files that meet
+            the selected lamp status.
 
         Raises:
             ValueError:
@@ -161,34 +189,30 @@ class ShaneKastSpectrograph(spectrograph.Spectrograph):
                                             if k in dome_lamp_stat]), axis=0)
         raise ValueError('No implementation for status = {0}'.format(status))
 
+
 class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
     """
     Child to handle Shane/Kast blue specific code
     """
-    def __init__(self):
-        # Get it started
-        super(ShaneKastBlueSpectrograph, self).__init__()
-        self.spectrograph = 'shane_kast_blue'
-        self.camera = 'KASTb'
-        # Uses timeunit from parent class
-        # Uses default primary_hdrext
-        self.sky_file = 'sky_kastb_600.fits'
+
+    name = 'shane_kast_blue'
+    camera = 'KASTb'
+    supported = True
 
     def get_detector_par(self, hdu, det):
         """
-        Return a DectectorContainer for the current image
+        Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`):
-                HDUList of the image of interest.
-                Ought to be the raw file, or else..
-            det (int):
+            hdu (`astropy.io.fits.HDUList`_):
+                The open fits file with the raw image of interest.
+            det (:obj:`int`):
+                1-indexed detector number.
 
         Returns:
-            :class:`pypeit.images.detector_container.DetectorContainer`:
-
+            :class:`~pypeit.images.detector_container.DetectorContainer`:
+            Object with the detector metadata.
         """
-
         # Detector 1
         detector_dict = dict(
             binning=self.get_meta_value(self.get_headarr(hdu), 'binning'),
@@ -217,12 +241,17 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
         # Return
         return detector
 
-    def default_pypeit_par(self):
+    @classmethod
+    def default_pypeit_par(cls):
         """
-        Set default parameters for Shane Kast Blue reductions.
+        Return the default parameters to use for this instrument.
+        
+        Returns:
+            :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
+            all of ``PypeIt`` methods.
         """
-        par = ShaneKastSpectrograph.default_pypeit_par()
-        par['rdx']['spectrograph'] = 'shane_kast_blue'
+        par = super().default_pypeit_par()
+
         par['flexure']['spectrum'] = os.path.join(resource_filename('pypeit', 'data/sky_spec/'),
                                                   'sky_kastb_600.fits')
         # 1D wavelength solution
@@ -233,7 +262,6 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
         par['calibrations']['wavelengths']['method'] = 'full_template'
         par['calibrations']['wavelengths']['n_first'] = 3
         par['calibrations']['wavelengths']['match_toler'] = 2.5
-        #par['calibrations']['wavelengths']['nonlinear_counts'] = self.detector[0]['nonlinear'] * self.detector[0]['saturation']
 
         # Set wave tilts order
         par['calibrations']['tilts']['spat_order'] = 3
@@ -245,25 +273,22 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
 
     def config_specific_par(self, scifile, inp_par=None):
         """
-        Modify the PypeIt parameters to hard-wired values used for
+        Modify the ``PypeIt`` parameters to hard-wired values used for
         specific instrument configurations.
 
-        .. todo::
-            Document the changes made!
-
         Args:
-            scifile (str):
+            scifile (:obj:`str`):
                 File to use when determining the configuration and how
                 to adjust the input parameters.
-            inp_par (:class:`pypeit.par.parset.ParSet`, optional):
+            inp_par (:class:`~pypeit.par.parset.ParSet`, optional):
                 Parameter set used for the full run of PypeIt.  If None,
                 use :func:`default_pypeit_par`.
 
         Returns:
-            :class:`pypeit.par.parset.ParSet`: The PypeIt paramter set
+            :class:`~pypeit.par.parset.ParSet`: The PypeIt parameter set
             adjusted for configuration specific parameter values.
         """
-        par = self.default_pypeit_par() if inp_par is None else inp_par
+        par = super().config_specific_par(scifile, inp_par=inp_par)
         # TODO: Should we allow the user to override these?
 
         if self.get_meta_value(scifile, 'dispname') == '600/4310':
@@ -280,12 +305,12 @@ class ShaneKastBlueSpectrograph(ShaneKastSpectrograph):
 
     def init_meta(self):
         """
-        Meta data specific to shane_kast_blue
-
-        Returns:
+        Define how metadata are derived from the spectrograph files.
 
+        That is, this associates the ``PypeIt``-specific metadata keywords
+        with the instrument-specific header cards using :attr:`meta`.
         """
-        super(ShaneKastBlueSpectrograph, self).init_meta()
+        super().init_meta()
         # Add the name of the dispersing element
         # dispangle and filter1 are not defined for Shane Kast Blue
 
@@ -298,26 +323,24 @@ class ShaneKastRedSpectrograph(ShaneKastSpectrograph):
     """
     Child to handle Shane/Kast red specific code
     """
-    def __init__(self):
 
-        # Get it started
-        super(ShaneKastRedSpectrograph, self).__init__()
-        self.spectrograph = 'shane_kast_red'
-        self.camera = 'KASTr'
+    name = 'shane_kast_red'
+    camera = 'KASTr'
+    supported = True
 
     def get_detector_par(self, hdu, det):
         """
-        Return a DectectorContainer for the current image
+        Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`):
-                HDUList of the image of interest.
-                Ought to be the raw file, or else..
-            det (int):
+            hdu (`astropy.io.fits.HDUList`_):
+                The open fits file with the raw image of interest.
+            det (:obj:`int`):
+                1-indexed detector number.
 
         Returns:
-            :class:`pypeit.images.detector_container.DetectorContainer`:
-
+            :class:`~pypeit.images.detector_container.DetectorContainer`:
+            Object with the detector metadata.
         """
         # Binning
         binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
@@ -358,7 +381,8 @@ class ShaneKastRedSpectrograph(ShaneKastSpectrograph):
         xo2_1 = xo1_2+1
         xo2_2 = xo1_2+nover
 
-        datasec = ['[:,{}:{}]'.format(x1_0, x1_1), '[:,{}:{}]'.format(x2_0,x2_1)]    # These are rows, columns on the raw frame, 1-indexed
+        # These are rows, columns on the raw frame, 1-indexed
+        datasec = ['[:,{}:{}]'.format(x1_0, x1_1), '[:,{}:{}]'.format(x2_0,x2_1)]
         oscansec = ['[:,{}:{}]'.format(xo1_1,xo1_2), '[:,{}:{}]'.format(xo2_1,xo2_2)]
 
         # Fill it up
@@ -367,28 +391,30 @@ class ShaneKastRedSpectrograph(ShaneKastSpectrograph):
 
         return detector_container.DetectorContainer(**detector_dict)
 
-
-    def default_pypeit_par(self):
+    @classmethod
+    def default_pypeit_par(cls):
         """
-        Set default parameters for Shane Kast Red reductions.
+        Return the default parameters to use for this instrument.
+        
+        Returns:
+            :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
+            all of ``PypeIt`` methods.
         """
-        par = ShaneKastSpectrograph.default_pypeit_par()
-        par['rdx']['spectrograph'] = 'shane_kast_red'
+        par = super().default_pypeit_par()
 
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['NeI','HgI','HeI','ArI']
-        #par['calibrations']['wavelengths']['nonlinear_counts'] = self.detector[0]['nonlinear'] * self.detector[0]['saturation']
 
         return par
 
     def init_meta(self):
         """
-        Meta data specific to shane_kast_red
-
-        Returns:
+        Define how metadata are derived from the spectrograph files.
 
+        That is, this associates the ``PypeIt``-specific metadata keywords
+        with the instrument-specific header cards using :attr:`meta`.
         """
-        super(ShaneKastRedSpectrograph, self).init_meta()
+        super().init_meta()
         # Add the name of the dispersing element
         # dispangle is not defined for Shane Kast Blue
 
@@ -399,45 +425,80 @@ class ShaneKastRedSpectrograph(ShaneKastSpectrograph):
 
     def configuration_keys(self):
         """
-        Return the metadata keys that defines a unique instrument
+        Return the metadata keys that define a unique instrument
         configuration.
 
-        This list is used by :class:`pypeit.metadata.PypeItMetaData` to
+        This list is used by :class:`~pypeit.metadata.PypeItMetaData` to
         identify the unique configurations among the list of frames read
         for a given reduction.
 
         Returns:
-
-            list: List of keywords of data pulled from meta
+            :obj:`list`: List of keywords of data pulled from file headers
+            and used to constuct the :class:`~pypeit.metadata.PypeItMetaData`
+            object.
         """
-        cfg_keys = super(ShaneKastRedSpectrograph, self).configuration_keys()
-        # Add grating tilt
-        return cfg_keys+['dispangle']
+        return super().configuration_keys() + ['dispangle']
+
+
+    def config_specific_par(self, scifile, inp_par=None):
+        """
+        Modify the PypeIt parameters to hard-wired values used for
+        specific instrument configurations.
+
+        .. todo::
+            Document the changes made!
+
+        Args:
+            scifile (str):
+                File to use when determining the configuration and how
+                to adjust the input parameters.
+            inp_par (:class:`pypeit.par.parset.ParSet`, optional):
+                Parameter set used for the full run of PypeIt.  If None,
+                use :func:`default_pypeit_par`.
+
+        Returns:
+            :class:`pypeit.par.parset.ParSet`: The PypeIt paramter set
+            adjusted for configuration specific parameter values.
+        """
+        par = self.default_pypeit_par() if inp_par is None else inp_par
+        # TODO: Should we allow the user to override these?
+
+        if self.get_meta_value(scifile, 'dispname') == '300/7500':
+            # TODO -- Note in docs that a NoNe solution is available too
+            par['calibrations']['wavelengths']['method'] = 'full_template'
+            par['calibrations']['wavelengths']['reid_arxiv'] = 'shane_kast_red_300_7500.fits'
+            # Add CdI
+            par['calibrations']['wavelengths']['lamps'] = ['NeI', 'HgI', 'HeI', 'ArI', 'CdI']
+        else:
+            pass
+        # Return
+        return par
+
+
 
 class ShaneKastRedRetSpectrograph(ShaneKastSpectrograph):
     """
     Child to handle Shane/Kast red specific code
     """
-    def __init__(self):
-        # Get it started
-        super(ShaneKastRedRetSpectrograph, self).__init__()
-        self.spectrograph = 'shane_kast_red_ret'
-        # WARNING: This is not unique wrt ShaneKastRed...
-        self.camera = 'KASTr'
+    name = 'shane_kast_red_ret'
+    # WARNING: This camera name is not unique wrt ShaneKastRed...
+    camera = 'KASTr'
+    supported = True
+    comment = 'Red reticon'
 
     def get_detector_par(self, hdu, det):
         """
-        Return a DectectorContainer for the current image
+        Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`):
-                HDUList of the image of interest.
-                Ought to be the raw file, or else..
-            det (int):
+            hdu (`astropy.io.fits.HDUList`_):
+                The open fits file with the raw image of interest.
+            det (:obj:`int`):
+                1-indexed detector number.
 
         Returns:
-            :class:`pypeit.images.detector_container.DetectorContainer`:
-
+            :class:`~pypeit.images.detector_container.DetectorContainer`:
+            Object with the detector metadata.
         """
         # Binning
         binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
@@ -464,45 +525,50 @@ class ShaneKastRedRetSpectrograph(ShaneKastSpectrograph):
 
         return detector_container.DetectorContainer(**detector_dict)
 
-    def default_pypeit_par(self):
+    @classmethod
+    def default_pypeit_par(cls):
         """
-        Set default parameters for Shane Kast Red Ret reductions.
+        Return the default parameters to use for this instrument.
+        
+        Returns:
+            :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
+            all of ``PypeIt`` methods.
         """
-        par = ShaneKastSpectrograph.default_pypeit_par()
-        par['rdx']['spectrograph'] = 'shane_kast_red_ret'
+        par = super().default_pypeit_par()
 
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['NeI', 'HgI', 'HeI', 'ArI']
-        #par['calibrations']['wavelengths']['nonlinear_counts'] = self.detector[0]['nonlinear'] * self.detector[0]['saturation']
+        par['calibrations']['wavelengths']['rms_threshold'] = 0.20
         par['calibrations']['wavelengths']['sigdetect'] = 5.
 
         return par
 
-    def check_header(self, headers):
-        """
-        Check headers match expectations for a Shane Kast Red Ret
-        exposure.
-
-        See also
-        :func:`pypeit.spectrographs.spectrograph.Spectrograph.check_headers`.
-
-        Args:
-            headers (list):
-                A list of headers read from a fits file
-        """
-        expected_values = {   '0.NAXIS': 2,
-                            '0.DSENSOR': 'Ret 400x1200' }
-        super(ShaneKastRedRetSpectrograph, self).check_headers(headers,
-                                                               expected_values=expected_values)
+# TODO: out-of-date
+#    def check_header(self, headers):
+#        """
+#        Check headers match expectations for a Shane Kast Red Ret
+#        exposure.
+#
+#        See also
+#        :func:`pypeit.spectrographs.spectrograph.Spectrograph.check_headers`.
+#
+#        Args:
+#            headers (list):
+#                A list of headers read from a fits file
+#        """
+#        expected_values = {   '0.NAXIS': 2,
+#                            '0.DSENSOR': 'Ret 400x1200' }
+#        super(ShaneKastRedRetSpectrograph, self).check_headers(headers,
+#                                                               expected_values=expected_values)
 
     def init_meta(self):
         """
-        Meta data specific to shane_kast_blue
-
-        Returns:
+        Define how metadata are derived from the spectrograph files.
 
+        That is, this associates the ``PypeIt``-specific metadata keywords
+        with the instrument-specific header cards using :attr:`meta`.
         """
-        super(ShaneKastRedRetSpectrograph, self).init_meta()
+        super().init_meta()
         # Add the name of the dispersing element
         # dispangle and filter1 are not defined for Shane Kast Blue
 
